// Craig S. Kaplan
//
// A variation that I didn't show in class.  Direct manipulation
// of an infinite grid of coloured tiles, with colours chosen 
// according to multiple calls to the noise() function.  Use the
// comma and period keys to zoom in and out on the noise function.

// This sketch also plays a clever trick.  It really does all its
// work at the pixel level and then scales the whole thing up 
// using geometric context, thus avoiding a bit of the difficult
// math in FullGridViz.

float tx = 0;
float ty = 0;
float fac = 0.25;

float cnoise( float a, float b, float c )
{
  return 255.0 * noise( a*fac, b*fac, c );
}

// Draw a square grid of tiles, translated by (tx,ty).
// Choose tile colours based on the underlying noise() function
// at those locations, which demonstrates that noise behaves
// consistently.  A useful tool in many sketches.  Just don't 
// zoom out too much (with the , and . keys) or you'll discover
// that the effective range of the noise() function isn't as 
// large as you'd like.
void drawTiles( float tx, float ty )
{
 float xmin = floor( tx );
 float ymin = floor( ty );
 for( int row = 0; row < 11; ++row ) {
   for( int col = 0; col < 11; ++col ) {
     color c = color( cnoise(xmin+col,ymin+row,0),
       cnoise(xmin+col,ymin+row,1),
       cnoise(xmin+col,ymin+row,2) );
     fill( c );
     rect( xmin-tx+col, ymin-ty+row, 1, 1 );
   }
 }   
}

void keyPressed()
{
  if( key == ',' ) {
    fac /= 2.0;
  } else if( key == '.' ) {
    fac *= 2.0;
  }
}

void setup()
{
  size( 500, 500 );
  noStroke();
}

void draw()
{
  background( 255 );
  pushMatrix();
  scale( 50 );
  drawTiles( tx, ty );
  popMatrix();
}

void mouseDragged()
{
  tx += (pmouseX-mouseX)/50.0;
  ty += (pmouseY-mouseY)/50.0;
}