// Craig S. Kaplan
//
// Fake Noise: demonstrate something that behaves a little
// bit like the noise() function, but looking under the hood
// so that you can get some idea of how it might work.
// noise() is a matheatical operation that always returns
// the same value for the same input x, and we can simulate
// that by, e.g., adding together a bunch of fixed sine waves.

// Amount of translation for direct manipulation purposes.
float dx;

void setup()
{
  size( 600, 200 );
  dx = 0;
}

float myFunc( float x )
{
  // Set a random seed.  Remember that this means that all
  // the floating point numbers generated below are exactly
  // the same every time you call myFunc().
  randomSeed( 98424 );
  
  // Add together 100 sine waves of different sizes and
  // shapes.
  float tot = 0.0;
  float weights = 0.0;
  for( int idx = 0; idx < 100; ++idx ) {
    float w = random( 10 );
    tot += sin( x / random( 1, 50 ) + random( PI ) ) * w;
    weights += w;
  }
  return map( tot, -weights, weights, 0, height );
}

void draw()
{
  background( 255 );
  strokeWeight( 2 );
  stroke( 255, 0, 0 );
  noFill();
  
  beginShape();
  for( int x = 0; x < 600; ++x ) {
    vertex( x, myFunc( x - dx ) );
  }
  endShape();
}

void mouseDragged()
{
  dx += mouseX - pmouseX;
}