// Craig S. Kaplan
//
// A fairly advanced sketch that demonstrates how to draw only
// as much of a grid as you need, but still keep track of "fake"
// coordinates that will later support direct manipulation.  This
// is beyond the level of math I'd expect from CS 106 students,
// but it's a very useful tool for understanding how we solved
// the direct manipulation problem in the context of the 10 Print
// sketch.  Think of it more as an in-class visualization tool.

// This isn't too far from the typical methods that power 2D
// tile-based video game like a dungeon crawler, where you can walk
// in every direction.

// Translation amounts for direct manipulation.
int dx;
int dy;

void setup()
{
  size( 700, 700 );
}

void draw()
{
  background( 255 );
  noFill();
  strokeWeight( 1 );
  stroke( 255, 0, 0 );
  
  // Where to start drawing the grid (the nearest starting
  // point above and to the left of the grid).
  int start_y = dy % 50;
  int start_x = dx % 50;
  
  // We need a small correction here to get this perfect.  The
  // problem is that the % operator works slightly different for
  // negative numbers, so we do something different here for 
  // negative versus positive.  Don't worry about it too much.
  if( dx > 0 ) {
    start_x -= 50;
  }
  if( dy > 0 ) {
    start_y -= 50;
  }
  
  // We *usually* need to draw one extra row and column.  
  // There's no point having a special case to limit
  // ourselves to 10 rows and/or 10 columns -- 11x11 is
  // already efficient enough.  (Q: when can we replace
  // either of these 11s by 10s?)
  for( int y = 0; y < 11; ++y ) {
    for( int x = 0; x < 11; ++x ) {
      fill( 255, 0, 0 );
      // Draw fake computed coordinates in each grid cell.
      // Notice that these fake coordinates act as a perfect
      // simulation of unbounded direct manipulation.
      text( "(" + (x-dx/50) + "," + (y-dy/50) + ")", 100 + x*50 + start_x, 100 + y*50 + start_y + 12 );
      noFill();
      rect( 100 + x*50 + start_x, 100 + y*50 + start_y, 50, 50 ); 
    }
  }
  
  noFill();
  stroke( 0 );
  strokeWeight( 3 );
  rect( 100, 100, 500, 500 );
}

void mouseDragged()
{
  dx += mouseX - pmouseX;
  dy += mouseY - pmouseY;
}