// Craig S. Kaplan
// This was Assignment 07 in the Winter 2017 offering of the course
// (see https://www.student.cs.uwaterloo.ca/~cs106/Winter2017/assignments/a07/).
// It's a really nice demonstration of noise functions, so I'm including
// it here.

// A global variable to hold the SVG drawing of the rover.
PShape rover;

void setup()
{
  size( 800, 600 );
  rover = loadShape( "rover.svg" );
}

// I'm going build up an abstract function that can draw
// generic terrains based on noise().  The idea is that
// in draw(), I can just call that function three times to
// get the background mountains, the foreground mountains,
// and the ground.  By passing in different combinations
// of parameters, I'll get different smoothnesses and parallax
// scrolling built in.

// Basically a wrapper around the noise() function with a lot
// of tunable parameters.  Useful to separate this function out
// from the function that comes next, so that we can easily 
// compute the vertical locations of the rover wheels.
//
// x: where in the sketch window you want the sample to be.
// xsc: horizontal scaling (smaller number means smoother terrain)
// xoffs: translation of this terrain.  Lets the terrain move.
// ymin, ymax: vertical range for the output.
// layer: choose from among different noise() functions by
//        sampling in a second dimension.
float getNoise( float x, float xsc, float xoffs,
  float ymin, float ymax, float layer )
{
  return map( noise( (x+xoffs) * xsc, layer ), 0, 1, ymin, ymax );
}

// Use advanced shapes to draw a complete landscape.  The parameters
// are as they are for the previous helper function, except that there's
// also a fill colour for the terrain.
void drawLandscape( color c, float xsc, float xoffs,
  float ymin, float ymax, float layer )
{
  noStroke();
  fill( c );
  beginShape();
  for( int x = -4; x < width + 4; x += 2 ) {
    // This is the key step. The "x" loop iterator is a fixed
    // set of sample locations in the sketch window.  That's
    // mapped to positions in the noise() function using the
    // remaining arguments to getNoise().
    vertex( x, getNoise( x, xsc, xoffs, ymin, ymax, layer ) );
  }
  vertex( width + 3, height + 1 );
  vertex( -4, height + 1 );
  endShape( CLOSE );
}

// A wheel is just two concentric discs.
void drawWheel( float x, float y, float r )
{
  noStroke();
  fill( 0 );
  ellipse( x, y, 2*r, 2*r );
  fill( 128 );
  ellipse( x, y, r, r );
}

// Do the actual work of drawing the scene.
void draw()
{  
  background( 0 );
  
  // I experimented with a few different ways to translate
  // the terrains, but in the end a simple measure based on
  // frame count works fine.  The fact that I define a float
  // variable to be equal to frameCount is a vestige of the
  // earlier versions of this code.
  float tx = frameCount;

  // Background mountains
  drawLandscape( #009898, 1.0 / 50.0, tx, 150, 350, 100 );
  // Foreground mountains
  drawLandscape( #04de46, 1.0 / 150.0, tx * 1.713123, 320, 450, 120 );
  // Ground
  drawLandscape( #ff9543, 1.0 / 40.0, tx * 4.385, 
    400 + rover.height, 400 + rover.height + 30, 140 );
    
  // The x locations in the sketch window for the rover's wheels.
  // (This could probably be a global variable, since it doesn't
  // change).
  float[] xs = {
    200 + 0.2 * rover.width,
    200 + 0.4 * rover.width,
    200 + 0.8 * rover.width };
  float[] ys = new float[3];
  
  // Compute the y values of the ground terrain for the corresponding
  // x values above.  Do this in a preliminary steps so that we can
  // re-use the values -- once to draw the wheels, and once to get
  // their average for positioning the rover.
  for( int idx = 0; idx < 3; ++idx ) {
    ys[idx] = getNoise( xs[idx], 1.0 / 40.0, tx * 4.385,
      400 + rover.height, 400 + rover.height + 30, 140 );
  }
  
  // Draw the rover.
  float yavg = (ys[0]+ys[1]+ys[2])/3.0;
  shape( rover, 200, yavg - rover.height - 20 );
  
  // Finally, draw the three wheels.
  drawWheel( xs[0], ys[0] - 15, 15 ); 
  drawWheel( xs[1], ys[1] - 15, 15 );
  drawWheel( xs[2], ys[2] - 15, 15 );
}