// Craig S. Kaplan
//
// Add direct manipulation to the 10 Print sketch.  This requires
// a fair amount of reworking, most importantly the use of noise()
// instead of random() to generate the choices of slashes or
// backslashes.  The result is an infinite field of slashes and
// backslashes that can be explored via direct manipulation.

float bias;
int tx;
int ty;

import controlP5.*;

ControlP5 ui;
Slider slider;

void setup()
{
  size( 800, 800 );
  bias = 0.5;

  ui = new ControlP5( this );
  slider = ui.addSlider( "Bias" )
    .setSize( 700, 30 )
    .setPosition( 10, 10 )
    .setRange( 0, 1 )
    .setValue( 0.5 );

  tx = 0;
  ty = 0;
}

void draw()
{
  background( 255 );
  strokeWeight( 15 );
  noFill();

  int start_y = (ty % 40);
  int start_x = (tx % 40);
  
  if( tx > 0 ) {
    tx -= 40;
  }
  if( ty > 0 ) {
    ty -= 40;
  }

  for ( int row = 0; row < 21; ++row ) {
    for ( int col = 0; col < 21; ++col ) {
      float x = start_x + col * 40;
      float y = start_y + row * 40;

      float v = noise( (x - tx) / 40.0, (y - ty) / 40.0 );

      if ( v < bias ) {
        // Draw a line from NW to SE
        line( x, y, x + 40, y + 40 );
      } else {
        // Draw a line from NE to SW
        line( x + 40, y, x, y + 40 );
      }
    }
  }
}

void controlEvent( ControlEvent ce )
{
  if ( ce.isFrom( slider ) ) {
    bias = slider.getValue();
  }
}

void mouseDragged()
{
  // Create a "dead zone" at the top of the sketch
  // so that you can drag the slider without performing
  // direct manipulation on the grid.  Otherwise, *both*
  // will happen.
  if ( mouseY > 100 ) {
    tx += mouseX - pmouseX;
    ty += mouseY - pmouseY;
  }
}