// Craig S. Kaplan
//
// A somewhat more complicated sketch that uses noise() to generate a 
// function that warps the plane.  Then it applies that warp to an image.
// Kind of like a funhouse mirror demonstration.  You can play with this
// sketch very easily by running it and moving the sliders around.
// It would be useful to read through it and figure out how it works.

import controlP5.*;

PImage img;
PImage warped;

ControlP5 ui;

Slider xy_scaling;
Slider dist_scaling;
Slider phase;

float cx;
float cy;

void setup()
{
  size( 400, 700 );
  img = loadImage( "leo.jpg" );
  img.resize( 400, 0 );
  img.loadPixels();

  ui = new ControlP5( this );
  xy_scaling = ui.addSlider( "XY Scaling" )
    .setRange( 0.0, 0.04 )
    .setSize( 300, 20 )
    .setPosition( 5, 5 );
  dist_scaling = ui.addSlider( "Distance Scaling" )
    .setRange( 0.0, 400.0 )
    .setSize( 300, 20 )
    .setPosition( 5, 30 );
  phase = ui.addSlider( "Phase" )
    .setRange( 0.0, 1.0 )
    .setSize( 300, 20 )
    .setPosition( 5, 55 );

  warped = createImage( img.width, img.height, 0 );
  controlEvent( null );

  cx = 0.0;
  cy = 0.0;
}

color readImg( float xx, float yy )
{
  int x = int( xx ) % img.width;
  int y = int( yy ) % img.height;

  if ( x < 0 ) {
    x += img.width;
  }
  if ( y < 0 ) {
    y += img.height;
  }

  return img.pixels[ y*img.width + x ];
}

PVector getOffsetVector( int x, int y )
{
  float xys = xy_scaling.getValue();
  float d = dist_scaling.getValue();
  float p = phase.getValue();

  float dx = d*(noise( x*xys, y*xys, p ) - 0.5);
  float dy = d*(noise( x*xys, y*xys, p + 10.0 ) - 0.5);

  return new PVector( dx, dy );
}

void draw()
{
  background( 50 );

  image( img, 0, 90 );
  image( warped, 0, height - warped.height );

  stroke( 255, 0, 0 );
  strokeWeight( 1.5 );

  float y = cy - (height-img.height);
  PVector vec = getOffsetVector( int(cx), int( y ) );
  float nx = cx + vec.x;
  float ny = y + vec.y;

  line( cx, height-img.height+y, nx, 90+ny );
}

void controlEvent( ControlEvent ce )
{
  if ( warped != null ) {
    warped.loadPixels();
    for ( int y = 0; y < img.height; ++y ) {
      for ( int x = 0; x < img.width; ++x ) {
        PVector vec = getOffsetVector( x, y );
        warped.pixels[y*img.width+x] = readImg( x + vec.x, y + vec.y );
      }
    }
    warped.updatePixels();
  }
}

void mousePressed()
{
  if( mouseY > (height-img.height) ) {
    cx = mouseX;
    cy = mouseY;
  }
}